/* Jogo de damas.
Comunicacao entre controle das pecas e modelo virtual, criacao dos objetos

Teclas com funcoes especiais: F1 - seleciona primeira peca viva,
F2 - mostra tabuleiro logico, F3 - edita tabuleiro logico,
F4 - repinta pecas do computador                           */



#include <bios.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "defs.h"

#include <dpmi.h>

#include "teclado.h"

#include "sr.h"

#include "arquivo.h"

#include "arq3d.h"

#include "grvd.h"

#include "solido.h"
#include "gr.h"
#include "transf.h"

#include "universo.h"

#include "contrsol.h"

#include "contruni.h"

#include "render.h"

#include "controle.h"

#include "imagem.h"

#include "damas.h"

#include "depura.h"



// para calcular frames por segundo
#include <time.h>
clock_t
	stCLI_JOGO,
	stCLF_JOGO,
	stCLI_FRAM,
	stCLF_FRAM,
   stCLTOT_FR = 0;
BOOL
	bINICIOU_CONTAGEM = FALSE;
UINT
	uFRAMES = 0;



/* construtor - inicia sistema 3D
**********************************************************/
CDamas::CDamas ( char* szArq )
{
   // define fonte de iluminacao
	m_oRender.mAtrIlum ( 1, 1, 2 );

   // carrega tabuleiro e pecas
   CArquivo3D
   	oArq ( szArq, m_oJogo );

	// define camera
   m_oSRC.mAtrOrigem ();
	m_oSRC.m_nXO = 0;
	m_oSRC.m_nYO = 0;
	m_oSRC.m_nZO = 1200;

   // angulos de rotacao da camera relativos ao SRU
   BYTE
		m_yAngXY = 0,
		m_yAngYZ = 0;

   // define solido temporario (para cada frame)
   m_oS.mCriaCompativel ( (CSolido&) m_oJogo );

   // controle do modelo do tabuleiro
   m_oCT.mAtrObjeto ( m_oJogo );

   // controle do solido usado a cada frame
   m_oCS.mAtrObjeto ( m_oS );

   // objeto que serah renderisado
	m_oRender.mAtrObjeto ( m_oS );

   // inicia geracao de numeros aleatorios
	srandom ( (int) biostime ( 0, 0 ) );
}



/* Move uma peca para a posicao definida no tabuleiro
uPeca deve estar entre 1 e 24
**********************************************************/
void CDamas::mMove ( UINT16 uPeca, INT16 nPosLin, INT16 nPosCol )
{
   m_oTransf.mTranslada ( UNPOS * nPosCol, 0, -UNPOS * nPosLin, m_oJogo, m_oCT.mRetParticao ( uPeca ) );
}



/* Move uma peca para a posicao definida no tabuleiro fazendo
uma animacao: peca transladando.
uPeca deve estar entre 1 e 24
- tempo de animacao depende da velocidade de execucao do programa e
 da distancia a ser transladada;
- distancia (em X e Y) depende do numero de posicoes a serem transladas em
 cada eixo
- granularidade: numero de unidades do SRU a serem transladadas num quadro
 dividido pelo total de posicoes (quadrados das pecas) a serem transladados.
- total de unidades do SRU transladadas em X: granularidade * nPosCol
- total de unidades do SRU transladadas em Y: granularidade * nPosLin
- numero de quadros em toda animacao: UNPOS / granularidade
Para manter a velocidade do objeto transladado mais ou menos constante
a granularidade eh diminuida pela metade se aumentam as posicoes a serem
transladadas alem de uma unidade.
Se a translacao eh de uma posicao do tabuleiro sao gerados 10 quadros
em que sao transladadas 10 unidades por quadro (se UNPOS = 100).
Se movimento for maior que uma posicao sao gerados mais quadros para
aumentar o tempo de animacao.
**********************************************************/
void CDamas::mMoveAnim ( UINT16 uPeca, INT16 nPosLin, INT16 nPosCol )
{
	// unidades do universo transladaveis por quadro
   INT
   	nGranularidade = 10;

	// Redefine quantas unidades transladar por quadro se linhas ou colunas
   // diferentes de uma unidade.
   // se deve transladar mais de uma linha dobra o numero de quadros
  	if ( abs ( nPosLin ) >= 2 || abs ( nPosCol ) >= 2 )
   	nGranularidade = 5;

   // define particao a ser transladada
   UINT
   	uParte = m_oCT.mRetParticao ( uPeca );

   // faz animacao
   for ( INT ncA = 0; ncA < UNPOS / nGranularidade; ncA ++ ) {
      // reposiciona conforme cada quadro
	   m_oTransf.mTranslada ( nGranularidade * nPosCol, 0, - nGranularidade * nPosLin, m_oJogo, uParte );
      // exibe quadros de animacao
      mExibe ();
   }
}



/* Altera modelo de uma peca para dama
**********************************************************/
void CDamas::mDamifica ( UINT16 uPeca )
{
   // faz escala em Y (estica)
   m_oCT.mEscalaMed ( 1, 2, 1, m_oJogo, m_oCT.mRetParticao ( uPeca ) );
}



/* Muda cor de uma peca
**********************************************************/
void CDamas::mAtrCor ( UINT16 uPeca, BYTE yCor )
{
   m_oCS.mAtrCorPols ( m_oCT.mRetPol1 ( uPeca ), POLSPECA, yCor );
}



/* Exibe tabuleiro durante o jogo conforme posicao da camera e
angulos de rotacao.
**********************************************************/
void CDamas::mExibe ()
{
	// opera solido
	// rotacao fixa no centro
	m_oTransf.mRotacionaAbs ( 0, m_yAngXZ, 0, m_oSRC );
	m_oSRC.mAplicaBase ( (CSolido&) m_oJogo, m_oS );
	m_oTransf.mRotaciona ( m_yAngXY, 0, m_yAngYZ, m_oS );
	m_oTransf.mTranslada ( m_oSRC.m_nXO, m_oSRC.m_nYO, m_oSRC.m_nZO, m_oS );

	// cria imagem
	m_oRender.mRenderPoligonos ();

   // conta frames gerados
   uFRAMES ++;
}



/* Muda a posicao das pecas no universo conforme tabelas
de posicoes de pecas e copia a tabela de pecas atual para
a anterior. bAnim indica se deve haver animacao (default FALSE)
**********************************************************/
void CDamas::mReposiciona ( BOOL bAnim )
{
	// atualiza posicao de cada peca no tabuleiro
   for ( INT16 ncPeca = 1; ncPeca < 25; ncPeca ++ ) {

   	// procura peca no tabuleiro anterior
      INT16
      	nLAnt,
         nCAnt;
      // verifica se peca estah no tabuleiro anterior, se nao encontrar entao
		// peca jah foi excluida, ignora peca
      // procura peca como peao e dama
      if ( m_oCtrl.mRetPosAnter ( ncPeca, nLAnt, nCAnt ) == FALSE &&
		     m_oCtrl.mRetPosAnter ( ncPeca | DAMA, nLAnt, nCAnt ) == FALSE )
         // passa para proxima peca
      	continue;

      // procura peca no tabuleiro atual
      INT16
      	nLAtu,
         nCAtu;
      // se nao encontrou peca na jogada atual entao estah morta ou eh dama
      if ( m_oCtrl.mRetPos ( ncPeca, nLAtu, nCAtu ) == FALSE )
      	// verifica se peca na jogada atual morreu procurando peca como dama
	      if ( m_oCtrl.mRetPos ( ncPeca | DAMA, nLAtu, nCAtu ) == FALSE ) {
            // nao encontrou peca como dama: peca morreu
	      	// move peca para cima
			   m_oTransf.mTranslada ( 0, -10 * UNPOS, 0, m_oJogo, m_oCT.mRetParticao ( ncPeca ) );
      	   // passa para proxima peca
      		continue;
         }
         else   // se peca na jogada atual eh dama
            // Verifica se na jogada anterior jah era dama: se sim obtem
            // posicao no tabuleiro, se nao transforma modelo 3D e obtem
            // posicao da peca no tabuleiro anterior.
            if ( m_oCtrl.mRetPosAnter ( ncPeca | DAMA, nLAnt, nCAnt ) == FALSE ) {

               // define posicao anterior da peca que vai virar dama
            	m_oCtrl.mRetPosAnter ( ncPeca, nLAnt, nCAnt );

			      // move peca para nova posicao
               // verifica se eh para animar e se peca se deslocou
			      if ( bAnim && nLAtu - nLAnt != 0 && nCAtu - nCAnt != 0 )
				      mMoveAnim ( ncPeca, nLAtu - nLAnt, nCAtu - nCAnt );
			      else
	      			mMove ( ncPeca, nLAtu - nLAnt, nCAtu - nCAnt );

		      	// transforma em dama
               mDamifica ( ncPeca );

	      	   // passa para proxima peca
   	   		continue;
            }

      // move peca para nova posicao
      // verifica se eh para animar e se peca se deslocou
      if ( bAnim && nLAtu - nLAnt != 0 && nCAtu - nCAnt != 0 )
	      mMoveAnim ( ncPeca, nLAtu - nLAnt, nCAtu - nCAnt );
      else
	      mMove ( ncPeca, nLAtu - nLAnt, nCAtu - nCAnt );
   }

   // copia tabela atual para anterior
   m_oCtrl.mAtualiza ();
}



/* Promove controle do teclado para usuario mover pecas e
tabuleiro. return TRUE - continua jogo, FALSE - jogo terminado
**********************************************************/
BOOL CDamas::mControleUsuario ()
{
   CTeclado
		oT;
	// espera todas teclas estarem soltas
	oT.mRetSoltas ();

	// angulo de rotacao da base do sistema de referencia de camera
	m_yAngXZ = 0;

	// peca selecionada para mover
	UINT16 static
		uPSel = NPECAS + 1;

   // verifica se a peca selecionada na jogada anterior ainda estah viva
   // enquanto nao for encontrada peca no tabuleiro (peca viva) procura
   INT16
      nL,
      nC,
		ncLimiteProc = 0;
	BOOL
		bFim = FALSE;
	while ( ! m_oCtrl.mRetViva ( m_oCtrl.mRetValorAtualPeca ( uPSel ) ) ) {

		// seleciona proxima peca
		uPSel ++;
		// verifica se selecionou peca existe, se nao seleciona primeira
		if ( uPSel == NPECAS + NPECAS + 1 )
			uPSel = NPECAS + 1;

		// verifica se terminou jogo (nao existem pecas)
		// verifica se jah procurou todas pecas possiveis
		if ( ncLimiteProc > NPECAS ) {
			bFim = TRUE;
			break;
		}
      else
			// conta pecas jah procuradas
			ncLimiteProc ++;
   }

	// seleciona peca, se encoutrou alguma, senao termina jogo
   if ( ! bFim )
	   mAtrCor ( uPSel, CORSEL );
   else
   	return FALSE;

	// enquanto usuario nao tiver feito uma jogada ou saido do programa
   do {

      // originalmente nao rotaciona base
		m_yAngXZ = 0;

      // permite alterar parametros do solido em relacao ao observador
      // teclou para cima
      if ( oT.mRetPress ( T_SOB + ESPECIAL ) )
		   if ( oT.mRetPress ( T_ALT_E ) || oT.mRetPress ( T_ALT_D ) )
      	   m_oSRC.m_nYO -= 2;   // teclou ALT + SOBE (deslisa para cima)
         else
      	   m_yAngYZ ++;
      // teclou para baixo
      if ( oT.mRetPress ( T_DES + ESPECIAL ) )
	      if ( oT.mRetPress ( T_ALT_E ) || oT.mRetPress ( T_ALT_D ) )
            m_oSRC.m_nYO += 2;   // teclou ALT + DESCE (deslisa para baixo)
         else
	         m_yAngYZ --;
      // teclou para direita
      if ( oT.mRetPress ( T_DIR + ESPECIAL ) )
	      if ( oT.mRetPress ( T_ALT_E ) || oT.mRetPress ( T_ALT_D ) )
            m_oSRC.m_nXO += 2;// teclou ALT + DIREITA (deslisa para direita)
         else
	         m_yAngXZ ++;
      // teclou para esquerda
      if ( oT.mRetPress ( T_ESQ + ESPECIAL ) )
	      if ( oT.mRetPress ( T_ALT_E ) || oT.mRetPress ( T_ALT_D ) )
            m_oSRC.m_nXO -= 2;// teclou ALT+ESQUERDA (deslisa para esquerda)
         else
	         m_yAngXZ --;
      // teclou HOME (giro antihorario)
      if ( oT.mRetPress ( T_HOME + ESPECIAL ) )
         m_yAngXY ++;
      // teclou PGUP (giro horario)
      if ( oT.mRetPress ( T_PG_UP + ESPECIAL ) )
         m_yAngXY --;
      // teclou A (para frente)
      if ( oT.mRetPress ( T_A ) )
         m_oSRC.m_nZO -= 2;
      // teclou Z (para tras)
      if ( oT.mRetPress ( T_Z ) )
         m_oSRC.m_nZO += 2;

		// permite selecao de pecas
      // teclou para cima - seleciona peca seguinte
      if ( oT.mRetPress ( T_SOB ) || oT.mRetPress ( T_DIR ) ){

			// altera cor da antiga peca selecionada para cor do time
			mAtrCor ( uPSel, CORUSUARIO );

			// seleciona proxima peca viva
         BOOL
         	bEscolhe = TRUE;
         while ( bEscolhe ) {
				// seleciona proxima peca
				uPSel ++;
				// verifica se selecionou peca existe, se nao seleciona primeira
				if ( uPSel == NPECAS + NPECAS + 1 )
					uPSel = NPECAS + 1;
            // verifica se peca estah no tabuleiro (viva)
            // se sim termina loop
            if ( m_oCtrl.mRetViva ( m_oCtrl.mRetValorAtualPeca ( uPSel ) ) )
            	bEscolhe = FALSE;
         }

			// espera ateh todas teclas serem soltas
			oT.mRetSoltas ();

			// altera cor da nova peca selecionada para cor de peca selecionada
			mAtrCor ( uPSel, CORSEL );
		}
      // teclou para baixo - seleciona peca anterior
      if ( oT.mRetPress ( T_DES ) || oT.mRetPress ( T_ESQ ) ) {

			// altera cor da antiga peca selecionada para cor do time
			mAtrCor ( uPSel, CORUSUARIO );

			// seleciona proxima peca anterior viva
         BOOL
         	bEscolhe = TRUE;
         while ( bEscolhe ) {
         	// seleciona peca anterior
				uPSel --;
				// verifica se selecionou peca existe, se nao seleciona primeira
				if ( uPSel == NPECAS )
					uPSel = NPECAS + NPECAS;
            // verifica se peca estah no tabuleiro (viva)
            // se sim termina loop
            if ( m_oCtrl.mRetViva ( m_oCtrl.mRetValorAtualPeca ( uPSel ) ) )
            	bEscolhe = FALSE;
         }

			// espera ateh todas teclas serem soltas
			oT.mRetSoltas ();

			// altera cor da nova peca selecionada para cor de peca selecionada
			mAtrCor ( uPSel, CORSEL );
		}

      // Define peca selecionada antes de testar jogadas do usuario,
      // peca selecionada pode mudar durante testes.
     	UINT16
        	uPAtual = uPSel;

      // testa movimentos de pecas do usuario
      INT16
      	nTecla = oT.mRetPress ();
      // se teclou alguma das direcoes possiveis faz jogada
      if ( nTecla == T_HOME || nTecla == T_PG_UP ||
		     nTecla == T_END  || nTecla == T_PG_DN ) {

         // faz jogada
      	BYTE
         	yResJogada = m_oCtrl.mJogadaUsuario ( nTecla, uPSel );

         // trata resultado da jogada
         if ( yResJogada == 0 )
            // jogada inacabada: espera todas teclas estarem soltas
	         oT.mRetSoltas ();
         else {
				// deseleciona peca
				mAtrCor ( uPSel, CORUSUARIO );

				if ( yResJogada == 1 ) {
					// continua jogo

					// atualiza tabuleiro (universo)
					mReposiciona ( TRUE );

               // exibe imagem do tabuleiro virtual
               mExibe ();

					return TRUE;
				}

            if ( yResJogada == 2 )
            	// termina jogo
            	return FALSE;
         }
      }

      // verifica se peca selecionada foi alterada pelos testes
      if ( uPSel != uPAtual ) {
      	// se sim altera cor da peca antiga para a cor do time
			mAtrCor ( uPAtual, CORUSUARIO );
         // altera cor da peca selecionada para cor de selecao
			mAtrCor ( uPSel, CORSEL );
      }

      // testa fim do programa
		// verifica se deve sair do jogo
	   if ( oT.mRetPress ( T_ESC ) )
			return FALSE;

      // testa se peca selecionada deve ser a primeira
      if ( oT.mRetPress ( T_F1 ) ) {
      	// se sim altera cor da peca antiga para a cor do time
			mAtrCor ( uPSel, CORUSUARIO );

         // seleciona nova peca
      	uPSel = 1 + NPECAS;

      	// altera cor da peca selecionada
			mAtrCor ( uPSel, CORSEL );
      }

      // exibe tabuleiro em modo texto
	   if ( oT.mRetPress ( T_F2 ) )
      	m_oCtrl.mMostra ();

      // edita tabela de pecas
      if ( oT.mRetPress ( T_F3 ) ) {

      	oT.mDesinstalaHandler ();

         do {

            m_oCtrl.mMostra ();
            printf ( "                             (7,7)\n" );
            INT
         	   nLin,
               nCol,
               nValor;
            printf ( "editar linha: \n" );
            scanf ( "%d", &nLin );
            printf ( "       coluna:\n" );
            scanf ( "%d", &nCol );
            printf ( "       valor: \n" );
            scanf ( "%d", &nValor );
            m_oCtrl.mAtrAtual ( nValor, nLin, nCol );

            printf ( "\nTecle espaco para terminar edicao.\n" );

         } while ( getch () != ' ' );

      	oT.mInstalaHandler ();
      }

      // repinta pecas do adversario se for teclado F4
      if ( oT.mRetPress ( T_F4 ) )
	      for ( UINT ucA = 1; ucA < NPECAS + 1; ucA ++ )
				mAtrCor ( ucA, CORCOMPUTADOR );

      // exibe dados sobre o ultimo movimento do computador
      if ( oT.mRetPress ( T_F5 ) ) {
      	gotoxy ( 1, 1 );
         printf ( "Ultimo movimento do computador\n" );
         printf ( "Profundidade: %d\n", m_oCtrl.m_yUltProf );
         printf ( "Peca: %d\n", m_oCtrl.m_yUltPeca );
         printf ( "Comando: %d\n", m_oCtrl.m_yUltComando );
         oT.mRetSoltas ();
      }

      // atualiza tabuleiro (universo)
   	mReposiciona ( TRUE );

      // exibe imagem do tabuleiro virtual
      mExibe ();

	} while ( ! bFim );
}



/* Executa jogo: telas de abertura e uma partida.
**********************************************************/
void CDamas::mExec ()
{
   CTeclado
   	oT;

   // posicao do tabuleiro no SRC
	m_oTransf.mRotacionaAbs ( 0, 0, 30, m_oSRC );

   // indica de quem eh a vez de jogar: usuario (TRUE) ou computador (FALSE)
   BOOL
   	bVezJogada = TRUE;

   // faz tabuleiro girar no inicio ateh ser escolhido quem comeca
   // cria mascara com mensagem inferior
   CImagem
   	oMascQuem ( "inic", FALSE );
   m_oRender.mCriaMascara ( oMascQuem, 4, MAXY - 11 );
   while ( 1 ) {

      // transforma solido para exibir
		m_oTransf.mRotacionaAbs ( 0, 254, 0, m_oSRC );
      m_oSRC.mAplica ( (CSolido&) m_oJogo, m_oS );

      // cria imagem
      m_oRender.mRenderPoligonos ();

      // testa teclado
      // se teclou H inicia jogo pelo humano
      if ( oT.mRetPress ( T_H ) ) {
			bVezJogada = TRUE;
         break;
      }
      // se teclou C inicia jogo pelo computador
      if ( oT.mRetPress ( T_C ) ) {
			bVezJogada = FALSE;
         break;
      }
      // se teclou ESC sai do programa
      if ( oT.mRetPress ( T_ESC ) )
	     	return;
   }
   // destroi mascara
   m_oRender.mDestroiMascara ();

   // faz tabuleiro girar ateh ser definido profundidade default para minimax
   // cria mascara com mensagem inferior
   CImagem
   	oMascProfIni ( "defmet", FALSE );
   INT16
   	nProfIni = 3;
   m_oRender.mCriaMascara ( oMascProfIni, 4, 0 );
   while ( 1 ) {

      // transforma solido para exibir
		m_oTransf.mRotacionaAbs ( 0, 254, 0, m_oSRC );
      m_oSRC.mAplica ( (CSolido&) m_oJogo, m_oS );

      // cria imagem
      m_oRender.mRenderPoligonos ();

      // testa teclado para definir profundidade default
      if ( oT.mRetPress ( T_0 ) ) {      // 0 indica jogadas aleatorias
			nProfIni = 0;
         break;
      }
      if ( oT.mRetPress ( T_1 ) ) {
			nProfIni = 1;
         break;
      }
      if ( oT.mRetPress ( T_2 ) ) {
      	nProfIni = 2;
         break;
      }
      if ( oT.mRetPress ( T_3 ) ) {
      	nProfIni = 3;
         break;
      }
      if ( oT.mRetPress ( T_4 ) ) {
      	nProfIni = 4;
         break;
      }
      if ( oT.mRetPress ( T_5 ) ) {
      	nProfIni = 5;
         break;
      }
      if ( oT.mRetPress ( T_6 ) ) {
      	nProfIni = 6;
         break;
      }
      // se teclou ESC sai do programa
      if ( oT.mRetPress ( T_ESC ) )
	     	return;
   }
   // destroi mascara
   m_oRender.mDestroiMascara ();

   // exibe instrucoes
   // carrega imagem
   CImagem
   	oI ( "infoi" );
   // prepara para exibir e exibe
   m_oRender.mAtrObjeto ( oI );
   m_oRender.mRenderImagem ();
   // aguarda ateh alguma tecla ser pressionada
   oT.mRetTeclas ( TRUE );
   // termina de exibir imagem
   m_oRender.mFimRenderImagem ();

   // reposiciona camera para jogo
	m_yAngXY = 0;
	m_yAngYZ = 0;
   m_oSRC.m_nZO = 825;
   m_oSRC.mAtrOrigemBase ();
	m_oTransf.mRotacionaAbs ( 0, 0, 30, m_oSRC );

   // cria primeira imagem do solido
   m_oSRC.mAplicaBase ( (CSolido&) m_oJogo, m_oS );
   m_oTransf.mTranslada ( m_oSRC.m_nXO, m_oSRC.m_nYO, m_oSRC.m_nZO, m_oS );
   m_oRender.mRenderPoligonos ();

   // promove jogadas ateh ser teclado ESC ou jogo ter acabado
   // define hora de inicio de jogo
	stCLI_JOGO = clock ();
	bINICIOU_CONTAGEM = TRUE;
	while ( 1 ) {

      // verifica de quem eh a jogada e permite jogar
      // se for usuario permite controle pelo teclado
      if ( bVezJogada == TRUE ) {

         // tempo de inicio da exibicao de imagens para usuario
      	stCLI_FRAM = clock ();

	      // verifica se foi teclado ESC, sai do programa quebrando loop
         if ( mControleUsuario () == FALSE ) {

            // tempo de termino da exibicao de imagens para usuario
           	stCLF_FRAM = clock ();
            stCLTOT_FR += stCLF_FRAM - stCLI_FRAM;

				break;
         }

         // tempo de termino da exibicao de imagens para usuario
        	stCLF_FRAM = clock ();
         stCLTOT_FR += stCLF_FRAM - stCLI_FRAM;
      }
      else
      	if ( m_oCtrl.mJogadaComputador ( nProfIni ) == FALSE )
         	break;

      bVezJogada = ! bVezJogada;
	}
   // define hora de termino de jogo
	stCLF_JOGO = clock ();

   // mostra tabuleiro estaticamente ateh ser teclado ESC
   // atualiza tabuleiro (universo)
  	mReposiciona ( TRUE );
   // ajusta camera para visao do inicio do jogo
   m_oSRC.mAtrOrigem ();
   m_oSRC.m_nZO = 1200;
	m_oTransf.mRotacionaAbs ( 0, 0, 30, m_oSRC );
   // exibe tabuleiro no fim do jogo ateh teclar ESC
   while ( ! oT.mRetPress ( T_ESC ) ) {

      // transforma solido para exibir
		m_oTransf.mRotacionaAbs ( 0, 254, 0, m_oSRC );
      m_oSRC.mAplica ( (CSolido&) m_oJogo, m_oS );

      // cria imagem
      m_oRender.mRenderPoligonos ();
   }
}



main ( INT nNumArgs, char *szArgs [] )
{
   // verifica se foi passado nome de arquivo, se nao usa nome default
   char
   	szArq [ 200 ];
   if ( nNumArgs == 1 )

   	strcpy ( szArq, "default" );
   else
   	strcpy ( szArq, szArgs [ 1 ] );

   // instala handler de teclado
   CTeclado
   	oT ( TRUE );

   // inicia jogo
	CDamas
		*poDamas = new CDamas ( szArq );

   // verifica se iniciou jogo, se sim executa
   if ( poDamas != NULL ) {

      // executa
   	poDamas -> mExec ();

      // termina
	   delete poDamas;
   }
   else
   	printf ( MSG_IMPOSSIVELEXECUTAR );

	// exibe mensagem final e resultado da contagem dos frames por segundo
	printf ( MSG_TITULO );
   if ( bINICIOU_CONTAGEM )
		printf ( MSG_TEMPO, (float) uFRAMES / ( (float) stCLTOT_FR / CLOCKS_PER_SEC ), (UINT) ( (float) ( stCLF_JOGO - stCLI_JOGO ) / CLOCKS_PER_SEC / 60 ) );
}
